"""
Gestor de configuración para guardar y cargar ajustes
"""

import json
import os

class ConfigManager:
    def __init__(self, config_file="maselli_simulator_config.json"):
        self.config_file = config_file
        self.default_config = {
            'connection_type': 'TCP',
            'com_port': 'COM8',
            'baud_rate': '115200',
            'ip_address': '10.1.33.18',
            'port': '8899',
            'adam_address': '01',
            'function_type': 'Sinusoidal',
            'min_brix_map': '0',
            'max_brix_map': '80',
            'cycle_time': '0.5',  # Cambiado de 'period' a 'cycle_time' para tiempo de ciclo completo
            'manual_brix': '10.0',
            # Configuración para NetCom
            'netcom_com_port': 'COM3',
            'netcom_baud_rate': '115200'
        }
    
    def save_config(self, config_data):
        """Guarda la configuración en archivo JSON"""
        try:
            with open(self.config_file, 'w') as f:
                json.dump(config_data, f, indent=4)
            return True
        except Exception as e:
            print(f"Error al guardar configuración: {e}")
            return False
    
    def load_config(self):
        """Carga la configuración desde archivo JSON"""
        if not os.path.exists(self.config_file):
            return self.default_config.copy()
        
        try:
            with open(self.config_file, 'r') as f:
                config = json.load(f)
            
            # Asegurarse de que todas las claves necesarias estén presentes
            for key, value in self.default_config.items():
                if key not in config:
                    config[key] = value
            
            # Migrar 'period' a 'cycle_time' si existe
            if 'period' in config and 'cycle_time' not in config:
                config['cycle_time'] = config['period']
                del config['period']
            
            return config
            
        except Exception as e:
            print(f"Error al cargar configuración: {e}")
            return self.default_config.copy()
    
    def get_connection_params(self, config, use_netcom_port=False):
        """Extrae los parámetros de conexión de la configuración"""
        conn_type = config.get('connection_type', 'Serial')
        
        if conn_type == "Serial":
            if use_netcom_port:
                # Para NetCom, usar el puerto COM físico dedicado
                return {
                    'port': config.get('netcom_com_port', 'COM3'),
                    'baud': int(config.get('netcom_baud_rate', '115200'))
                }
            else:
                return {
                    'port': config.get('com_port', 'COM3'),
                    'baud': int(config.get('baud_rate', '115200'))
                }
        else:
            return {
                'ip': config.get('ip_address', '192.168.1.100'),
                'port': int(config.get('port', '502'))
            }
    
    def validate_config(self, config):
        """Valida que la configuración tenga valores correctos"""
        errors = []
        
        # Validar dirección ADAM
        adam_address = config.get('adam_address', '')
        if len(adam_address) != 2:
            errors.append("La dirección ADAM debe tener exactamente 2 caracteres")
        
        # Validar rango de Brix
        try:
            min_brix = float(config.get('min_brix_map', '0'))
            max_brix = float(config.get('max_brix_map', '80'))
            if min_brix >= max_brix:
                errors.append("El valor mínimo de Brix debe ser menor que el máximo")
        except ValueError:
            errors.append("Los valores de Brix deben ser números válidos")
        
        # Validar tiempo de ciclo
        try:
            cycle_time = float(config.get('cycle_time', '1.0'))
            if cycle_time <= 0:
                errors.append("El tiempo de ciclo debe ser mayor que 0")
        except ValueError:
            errors.append("El tiempo de ciclo debe ser un número válido")
        
        # Validar puerto serie
        if config.get('connection_type') == 'Serial':
            com_port = config.get('com_port', '')
            if not com_port.upper().startswith('COM'):
                errors.append("El puerto COM debe tener formato 'COMx'")
            
            try:
                baud_rate = int(config.get('baud_rate', '9600'))
                if baud_rate <= 0:
                    errors.append("La velocidad de baudios debe ser mayor que 0")
            except ValueError:
                errors.append("La velocidad de baudios debe ser un número entero")
        
        # Validar configuración TCP/UDP
        else:
            try:
                port = int(config.get('port', '502'))
                if port <= 0 or port > 65535:
                    errors.append("El puerto debe estar entre 1 y 65535")
            except ValueError:
                errors.append("El puerto debe ser un número entero")
        
        return errors
